"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateFolderStructure = generateFolderStructure;
exports["default"] = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _fs = _interopRequireDefault(require("fs"));

var _https = _interopRequireDefault(require("https"));

var _unzipper = _interopRequireDefault(require("unzipper"));

var _rimraf = _interopRequireDefault(require("rimraf"));

var _queueAsync = _interopRequireDefault(require("queue-async"));

var _util = require("util");

var _replaceString = _interopRequireDefault(require("./replaceString"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function generateFolderStructure(_options, _callback) {
  var options = _objectSpread({}, _options);

  var repoName = options.type === 'mobile' ? 'fl-base-mobile-app' : 'fl-base-webapp';
  var repoZipUrl = "https://codeload.github.com/founderlab/".concat(repoName, "/zip/master");
  var zipFilename = "".concat(repoName, ".zip");
  var oldFolder = "".concat(repoName, "-master");
  var newFolder = options.name;

  var writer = _fs["default"].createWriteStream(zipFilename);

  var replace = [{
    filePath: "".concat(newFolder, "/shared/modules/app/containers/Navbar.js"),
    name: options.name
  }, {
    filePath: "".concat(newFolder, "/package.json"),
    name: options.name
  }, {
    filePath: "".concat(newFolder, "/.env"),
    name: options.name.toLowerCase().replace(/\W/g, '_')
  }];

  function callback(err) {
    var queue = new _queueAsync["default"](); // delete zip

    queue.defer(function (callback) {
      _fs["default"].access(zipFilename, _fs["default"].F_OK, function (err) {
        if (err) return callback(err);

        _fs["default"].unlink(zipFilename, function (err) {
          if (err) return callback(err);
          if (options.verbose) console.log('--Zip deleted.');
          callback(null);
        });
      });
    }); // delete old folder

    queue.defer(function (callback) {
      _fs["default"].access(oldFolder, _fs["default"].F_OK, function (err) {
        if (err) {
          // err means successfully renamed
          return callback(null);
        } // otherwise delete the old folder


        (0, _rimraf["default"])(oldFolder, function (err) {
          if (err) return callback(err);
          if (options.verbose) console.log('--Folder ' + oldFolder + ' deleted.');
          callback(new Error('Failed when renaming ' + oldFolder + ' to ' + newFolder + ', ' + newFolder + ' already exists.'));
        });
      });
    });

    if (err && err.message === "String replacement failure: ".concat(err)) {
      // delete new folder
      queue.defer(function (callback) {
        _fs["default"].access(newFolder, _fs["default"].F_OK, function (err) {
          if (err) return callback(err); // otherwise delete the old folder

          (0, _rimraf["default"])(newFolder, function (err) {
            if (err) return callback(err);
            if (options.verbose) console.log('--Folder ' + newFolder + ' deleted.');
            callback(new Error('Failed when modifying .env or package.json.'));
          });
        });
      });
    }

    queue["await"](_callback);
  } // download, unzipper, rename


  _https["default"].get(repoZipUrl, function (res) {
    res.on('data', function (d) {
      return writer.write(d);
    });
    res.on('end', function () {
      if (options.verbose) console.log('--Zip downloaded.');

      var stream = _fs["default"].createReadStream(zipFilename).pipe(_unzipper["default"].Extract({
        path: './'
      }));

      stream.on('close', function () {
        if (options.verbose) console.log('--Zip extracted to folder ' + oldFolder + '.');

        _fs["default"].rename(oldFolder, newFolder, function (err) {
          if (err) return callback(err);
          if (options.verbose) console.log('--Folder renamed to ' + newFolder + '.');
          var queue = new _queueAsync["default"](); // Set the app name in a few files

          replace.forEach(function (_ref) {
            var filePath = _ref.filePath,
                name = _ref.name;
            queue.defer(function (callback) {
              (0, _replaceString["default"])(filePath, /FounderLab_replaceme/g, name, function (err) {
                if (err) return callback(err);
                if (options.verbose) console.log('--' + filePath + ' modified.');
                callback();
              });
            });
          });
          queue["await"](callback);
        });
      });
    });
  });
}

var _default = (0, _util.promisify)(generateFolderStructure);

exports["default"] = _default;